package	jp.co.sds_corp.hibiya.action;

import jp.co.sds_corp.hibiya.filter.EventFilter;
import jp.co.sds_corp.hibiya.filter.IFilter;
import jp.co.sds_corp.hibiya.layout.ILayout;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;

/**
* 動作の基底クラスです。
* Valid属性をfalseに指定すると、無効になります。
* &lt;Layout&gt;要素はここに保持していますが
必須ではありません。
*/
public abstract class AbstractAction implements IAction
{
	private static final long serialVersionUID = 1L;
	ILayout	layout;
	boolean	valid = true;
	IFilter filter;

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		try {
			setLayout( createLayout( config.getChild( "Layout" )));
		} catch ( UnsupportedOperationException uex ) {
		}
		valid = config.getAttributes().getBoolean( "Valid", true );

		IConfiguration c = config.getChild( "Filter" );
		if ( c != null ) {
			filter = createFilter( c );
		}
		if ( filter == null ) {
			filter = new EventFilter();
		}
	}

	/**
	* イベントをフィルタするIFilterを生成します。
	* @return   生成されたIFilter
	*/
	protected IFilter createFilter( IConfiguration config ) throws Exception {
		IFilter ret = (IFilter )ConfigUtility.createInstance( config );
		ret.init( config );
		return ret;
	}

	/**
	* イベントをフォーマットするILayoutを生成します。
	*/
	protected ILayout createLayout( IConfiguration config ) throws Exception {
		if ( config == null ) {
			return null;
		}
		ILayout ret = (ILayout )ConfigUtility.createInstance( config );
		String fmt = config.getAttributes().get( "Format", null );
		if ( fmt == null ) {
			return null;
		}
		ret.setFormat( fmt );
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		IConfiguration c;
		config.setAttribute( "class", getClass().getName() );
		if ( layout != null ) {
			c = config.addChild( "Layout" );
			c.setAttribute( "class", layout.getClass().getName() );
			c.setAttribute( "Format", layout.getFormat() );
		}
		if ( !isValid()) {
			config.setAttribute( "Valid", "false" );
		}

		c = config.addChild( "Filter" );
		getFilter().saveConfig( c );
	}

	/**
	* 設定内容の文字列表現を取得します。
	*/                      
	public String paramString() {
		String ret = "";
		if ( !isValid()) {
			ret += "X ";
		}
		return ret;
	}

	/**
	* このアクションの文字列表現を返します。
	*/
	public String toString() {
		String ret = getClass().getName();
		if ( paramString() != null ) {
			ret += " " + paramString();
		}
		if ( getFilter() != null ) {
			ret += " F{" + getFilter() + "}";
		}
		if ( layout != null ) {
			ret += " L{" + layout.toString() + "}";
		}
		return ret;
	}

	/**
	* イベントをフィルタするIFilter(filter)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFilter( IFilter var ) {
		this.filter = var ;
	}
	/**
	* イベントをフィルタするIFilter(filter)を取得します。
	* @return	layoutの内容
	*/
	public IFilter getFilter() {
		return filter;
	}

	/**
	* この動作が有効であるかどうかを返します。
	*/
	public boolean isValid() {
		return valid;
	}

	/**
	* この動作の有効であるかどうかを設定します。
	* @param    var     有効フラグ
	*/
	public void setValid( boolean var ) {
		this.valid = var;
	}

	/**
	* イベントをフォーマットするILayout(layout)を設定します。
	* @param	var		設定するデータ
	*/
	public void setLayout( ILayout var ) {
		this.layout = var ;
	}
	/**
	* イベントをフォーマットするILayout(layout)を取得します。
	* @return	layoutの内容
	*/
	public ILayout getLayout() {
		return layout;
	}

}
